#ifndef __KYSEC_API_H__
#define __KYSEC_API_H__

#include <fcntl.h>
#include <dlfcn.h>
#include <syslog.h>

static int kysec_get_status()
{
	int kysec_status = 0, exectl_status = 0, status = 0;
	char buf[8] = {"\0"};
	memset(buf, 0x00, sizeof(buf));
	int kysec_fd = open("/sys/kernel/security/kysec/status", O_RDONLY);
	if (kysec_fd < 0)
		return -1;

	if (read(kysec_fd, buf, sizeof(buf)) < 0) {
		close(kysec_fd);
		return -1;
	}
	close(kysec_fd);
	sscanf(buf, "%d", &kysec_status);

	memset(buf, 0x00, sizeof(buf));
	int exectl_fd = open("/sys/kernel/security/kysec/exectl", O_RDONLY);
	if (exectl_fd < 0)
		return -1;

	if (read(exectl_fd, buf, sizeof(buf)) < 0) {
		close(exectl_fd);
		return -1;
	}
	close(exectl_fd);
	sscanf(buf, "%d", &exectl_status);

	if (2 == kysec_status && 1 == exectl_status)
		status = 1;//拦截且输出日志
	else if(4 == kysec_status && 1 == exectl_status)
		status = 2;//不拦截，但输出日志
	else
		status = 0;//不拦截

	return status;
}

/*
 * 1 检测通过， 0 检测失败
 */
static int kysec_check_api(char *path)
{
	void *handle;
	int (*check)(char *);
	char *error;
	int retval = 0;
	int status = 0;

	status = kysec_get_status();
	if(status <= 0)
		return 1;

	handle = dlopen("libkysec.so.0", RTLD_LAZY);
	if (!handle) {
		syslog(LOG_INFO, "Failed to open libkysec - %s", dlerror());
		return 1;
	}

	dlerror();

	check = (int (*)(char *)) dlsym(handle, "kysec_check_exec");
	error = dlerror();
	if (error) {
		syslog(LOG_INFO, "Failed to obtain address of symbol - %s", error);
		dlclose(handle);
		return 1;
	}

	retval = (*check)(path);
	dlclose(handle);
	return retval;
}

/* Kysec module check */
static int kysec_module_check(const wchar_t *mod)
{
	int ret = 0;
	size_t len = wcslen(mod);
	char *path = NULL;
	int status = 0;

	status = kysec_get_status();
	if(status <= 0)
		return 0;

	path = malloc(len + 4);
	if (NULL == path)
		return -1;
	memset(path, 0, len+4);

	wcstombs(path, mod, len);
	if (len < 4 || strcmp(path+len-3, ".py"))
		strncat(path, ".py", 4);

	if (kysec_check_api(path) == 0)
		ret = -2;

	free(path);
	return ret;
}

#endif
